using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Buses.Queries.GetBus;

public class GetBusQueryHandler :
    IRequestHandler<GetBusQuery, BusDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetBusQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<BusDto> Handle(
        GetBusQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.BusRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.Company,
            cancellationToken);

        _unitOfWork.Dispose();

        if (entity == null)
        {
            throw new NotFoundException();
        }

        return _mapper.Map<BusDto>(entity);
    }
}
