using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.Companies.Commands.UpdateCompany;

public class UpdateCompanyCommandAuthorizer :
    AbstractRequestAuthorizer<UpdateCompanyCommand>
{
    private readonly SessionUserService _sessionUserService;

    public UpdateCompanyCommandAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(UpdateCompanyCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
