using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Companies.Commands.UpdateCompany;

public class UpdateCompanyCommandHandler : 
    IRequestHandler<UpdateCompanyCommand, CompanyDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public UpdateCompanyCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<CompanyDto> Handle(
        UpdateCompanyCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.CompanyRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        entity.Name = request.Name;
        entity.LegalAddress = request.LegalAddress;
        entity.ContactEmail = request.ContactEmail;
        entity.ContactPhoneNumber = request.ContactPhoneNumber;

        entity = await _unitOfWork.CompanyRepository.UpdateOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<CompanyDto>(entity);
    }
}
