using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Countries.Queries.GetCountriesPage;

public class GetCountriesPageQueryHandler :
    IRequestHandler<GetCountriesPageQuery, PaginatedList<CountryDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetCountriesPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<CountryDto>> Handle(
        GetCountriesPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.CountryRepository.GetPageAsync(
            e => e.Name.ToLower().Contains(request.Search.ToLower()),
            request.PageNumber, request.PageSize, cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<CountryDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<CountryDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<CountryDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
