using cuqmbr.TravelGuide.Application.Common.FluentValidation;
using cuqmbr.TravelGuide.Application.Common.Services;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application.Identity.Accounts.Commands.AddAccount;

public class AddAccountCommandValidator : AbstractValidator<AddAccountCommand>
{
    public AddAccountCommandValidator(
        IStringLocalizer localizer,
        SessionCultureService cultureService)
    {
        RuleFor(v => v.Email)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .IsEmail()
            .WithMessage(localizer["FluentValidation.IsEmail"]);

        RuleFor(v => v.Password)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .MinimumLength(8)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MinimumLength"],
                    8))
            .MaximumLength(64)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MaximumLength"],
                    64));
    }
}
