using MediatR;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Domain.Enums;

namespace cuqmbr.TravelGuide.Application.Identity.Roles.Queries.GetRolesPage;

public class GetRolesPageQueryHandler :
    IRequestHandler<GetRolesPageQuery, PaginatedList<string>>
{
    public async Task<PaginatedList<string>> Handle(
        GetRolesPageQuery request,
        CancellationToken cancellationToken)
    {
        var roleStrings = IdentityRole.Enumerations.Select(e => e.Value.Name);

        var roleCount = roleStrings.Count();

        var filteredRoles = roleStrings
            .Where(r => r.Contains(request.Search))
            .ToArray();

        return new PaginatedList<string>(
            filteredRoles, roleCount, request.PageNumber, request.PageSize);
    }
}
