using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Regions.Queries.GetRegionsPage;

public class GetRegionsPageQueryHandler :
    IRequestHandler<GetRegionsPageQuery, PaginatedList<RegionDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetRegionsPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<RegionDto>> Handle(
        GetRegionsPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.RegionRepository.GetPageAsync(
            e =>
                (e.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.Country.Name.ToLower().Contains(request.Search.ToLower())) &&
                (request.CountryGuid != null
                    ? e.Country.Guid == request.CountryGuid
                    : true),
            e => e.Country,
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<RegionDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<RegionDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<RegionDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
