using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Trains.Queries.GetTrainsPage;

public class GetTrainsPageQueryHandler :
    IRequestHandler<GetTrainsPageQuery, PaginatedList<TrainDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetTrainsPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<TrainDto>> Handle(
        GetTrainsPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.TrainRepository.GetPageAsync(
            e =>
                (e.Number.ToLower().Contains(request.Search.ToLower()) ||
                e.Model.ToLower().Contains(request.Search.ToLower())) &&
                (request.CompanyGuid != null
                    ? e.Company.Guid == request.CompanyGuid
                    : true) &&
                (request.CapacityGreaterOrEqualThan != null
                    ? e.Capacity >= request.CapacityGreaterOrEqualThan
                    : true) &&
                (request.CapacityLessOrEqualThan != null
                    ? e.Capacity <= request.CapacityLessOrEqualThan
                    : true),
            e => e.Company,
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<TrainDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<TrainDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<TrainDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
