using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Commands.AddVehicleEnrollment;

public class AddVehicleEnrollmentCommandAuthorizer :
    AbstractRequestAuthorizer<AddVehicleEnrollmentCommand>
{
    private readonly SessionUserService _sessionUserService;

    public AddVehicleEnrollmentCommandAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(AddVehicleEnrollmentCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
