using cuqmbr.TravelGuide.Application.Common.FluentValidation;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Commands.AddVehicleEnrollment;

public class AddVehicleEnrollmentCommandValidator :
    AbstractValidator<AddVehicleEnrollmentCommand>
{
    public AddVehicleEnrollmentCommandValidator(
        IStringLocalizer localizer,
        SessionCultureService cultureService,
        SessionTimeZoneService timeZoneService)
    {
        RuleFor(v => v.DepartureTime)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"])
            .GreaterThanOrEqualTo(DateTimeOffset.Now)
                .WithMessage(
                    String.Format(
                        cultureService.Culture,
                        localizer["FluentValidation.GreaterThanOrEqualTo"],
                        DateTimeOffset.Now.ToOffset(timeZoneService.TimeZone.BaseUtcOffset)));

        RuleFor(v => v.Currency)
            .Must(c => Currency.Enumerations.ContainsValue(c))
            .WithMessage(
                String.Format(
                    localizer["FluentValidation.MustBeInEnum"],
                    String.Join(
                        ", ",
                        Currency.Enumerations.Values.Select(e => e.Name))));

        RuleFor(v => v.RouteAddressDetails.Count)
            .GreaterThanOrEqualTo(2)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    2));

        RuleForEach(v => v.RouteAddressDetails).ChildRules(rad =>
        {
            rad.RuleFor(rad => rad.RouteAddressGuid)
                .NotEmpty()
                .WithMessage(localizer["FluentValidation.NotEmpty"]);

            rad.RuleFor(rad => rad.TimeToNextAddress)
                .GreaterThanOrEqualTo(TimeSpan.Zero)
                .WithMessage(
                    String.Format(
                        cultureService.Culture,
                        localizer["FluentValidation.GreaterThanOrEqualTo"],
                        TimeSpan.Zero));

            rad.RuleFor(rad => rad.CostToNextAddress)
                .GreaterThanOrEqualTo(0)
                .WithMessage(
                    String.Format(
                        cultureService.Culture,
                        localizer["FluentValidation.GreaterThanOrEqualTo"],
                        0));

            rad.RuleFor(rad => rad.CurrentAddressStopTime)
                .GreaterThanOrEqualTo(TimeSpan.Zero)
                .WithMessage(
                    String.Format(
                        cultureService.Culture,
                        localizer["FluentValidation.GreaterThanOrEqualTo"],
                        TimeSpan.Zero));
        });

        RuleFor(v => v.VehicleGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);

        RuleFor(v => v.RouteGuid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);

        RuleFor(v => v.EmployeeGuids.Count)
            .GreaterThanOrEqualTo(1)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.GreaterThanOrEqualTo"],
                    1));

        RuleFor(v => v.EmployeeGuids)
            .IsUnique(g => g)
            .WithMessage(localizer["FluentValidation.IsUnique"]);

        RuleForEach(v => v.EmployeeGuids)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);
    }
}
