using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments;

public sealed class VehicleEnrollmentVehicleDto : IMapFrom<Vehicle>
{
    public string Uuid { get; set; }

    public string Type { get; set; }

    public string Number { get; set; }

    public string Model { get; set; }

    public short Capacity { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<Vehicle, VehicleEnrollmentVehicleDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.Type,
                opt => opt.MapFrom(s => s.VehicleType.Name))
            .ForMember(
                d => d.Number,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        if (s is Bus)
                        {
                            return ((Bus)s).Number;
                        }
                        else if (s is Aircraft)
                        {
                            return ((Aircraft)s).Number;
                        }
                        else if (s is Train)
                        {
                            return ((Train)s).Number;
                        }
                        else
                        {
                            throw new NotImplementedException();
                        }
                    }))
            .ForMember(
                d => d.Model,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        if (s is Bus)
                        {
                            return ((Bus)s).Model;
                        }
                        else if (s is Aircraft)
                        {
                            return ((Aircraft)s).Model;
                        }
                        else if (s is Train)
                        {
                            return ((Train)s).Model;
                        }
                        else
                        {
                            throw new NotImplementedException();
                        }
                    }))
            .ForMember(
                d => d.Capacity,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        if (s is Bus)
                        {
                            return ((Bus)s).Capacity;
                        }
                        else if (s is Aircraft)
                        {
                            return ((Aircraft)s).Capacity;
                        }
                        else if (s is Train)
                        {
                            return ((Train)s).Capacity;
                        }
                        else
                        {
                            throw new NotImplementedException();
                        }
                    }));
    }
}
