using cuqmbr.TravelGuide.Domain.Enums;

namespace cuqmbr.TravelGuide.Domain.Entities;

public class VehicleEnrollment : EntityBase
{
    public DateTimeOffset DepartureTime { get; set; }

    public Currency Currency { get; set; }


    public long VehicleId { get; set; }

    public Vehicle Vehicle { get; set; }


    public long RouteId { get; set; }

    public Route Route { get; set; }


    public ICollection<RouteAddressDetail> RouteAddressDetails { get; set; }

    public ICollection<Ticket> Tickets { get; set; }

    public ICollection<VehicleEnrollmentEmployee> VehicleEnrollmentEmployees { get; set; }


    public DateTimeOffset GetDepartureTime(long DepartureRouteAddressId)
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;

        if (DepartureRouteAddressId == firstRouteAddressId)
        {
            return DepartureTime;
        }


        var orderedRouteAddressDetails = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order);

        var timeToDeparture = TimeSpan.Zero;
        foreach (var routeAddressDetail in orderedRouteAddressDetails)
        {
            timeToDeparture = 
                timeToDeparture + routeAddressDetail.CurrentAddressStopTime;

            if (routeAddressDetail.Id == DepartureRouteAddressId)
            {
                break;
            }

            timeToDeparture =
                timeToDeparture += routeAddressDetail.TimeToNextAddress;
        }

        return DepartureTime + timeToDeparture;
    }

    public DateTimeOffset GetDepartureTime()
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;

        return GetDepartureTime(firstRouteAddressId);
    }

    public DateTimeOffset GetArrivalTime(long ArrivalRouteAddressId)
    {
        var orderedRouteAddressDetails = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order);

        var timeToDeparture = TimeSpan.Zero;
        foreach (var routeAddressDetail in orderedRouteAddressDetails)
        {
            if (routeAddressDetail.Id == ArrivalRouteAddressId)
            {
                break;
            }

            timeToDeparture =
                timeToDeparture +
                routeAddressDetail.TimeToNextAddress +
                routeAddressDetail.CurrentAddressStopTime;
        }

        return DepartureTime + timeToDeparture;
    }

    public DateTimeOffset GetArrivalTime()
    {
        var lastRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).Last().RouteAddressId;

        return GetArrivalTime(lastRouteAddressId);
    }

    public TimeSpan GetTravelTime(
        long DepartureRouteAddressId, long ArrivalRouteAddressId)
    {
        return
            GetArrivalTime(ArrivalRouteAddressId) - 
            GetDepartureTime(DepartureRouteAddressId);
    }

    public TimeSpan GetTravelTime()
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;
        var lastRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).Last().RouteAddressId;

        return GetTravelTime(firstRouteAddressId, lastRouteAddressId);
    }

    public TimeSpan GetTimeInStops(
        long DepartureRouteAddressId, long ArrivalRouteAddressId)
    {
        var orderedRouteAddressDetails = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order);

        var departureRouteAddressDetail = orderedRouteAddressDetails
            .Single(e => e.Id == DepartureRouteAddressId);

        var timeInStops = TimeSpan.Zero;
        foreach (var routeAddressDetail in orderedRouteAddressDetails)
        {
            if (routeAddressDetail.RouteAddress.Order <=
                departureRouteAddressDetail.RouteAddress.Order)
            {
                continue;
            }

            if (routeAddressDetail.Id == ArrivalRouteAddressId)
            {
                break;
            }

            timeInStops += routeAddressDetail.CurrentAddressStopTime;
        }

        return timeInStops;
    }

    public TimeSpan GetTimeInStops()
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;
        var lastRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).Last().RouteAddressId;

        return GetTimeInStops(firstRouteAddressId, lastRouteAddressId);
    }

    public int GetNumberOfStops(
        long DepartureRouteAddressId, long ArrivalRouteAddressId)
    {
        return
            RouteAddressDetails
                .OrderBy(e => e.RouteAddress.Order)
                .SkipWhile(e => e.Id != DepartureRouteAddressId)
                .TakeWhile(e => e.Id != ArrivalRouteAddressId)
                .Count() - 1;
    }

    public TimeSpan GetNumberOfStops()
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;
        var lastRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).Last().RouteAddressId;

        return GetTimeInStops(firstRouteAddressId, lastRouteAddressId);
    }

    public TimeSpan GetTimeMoving(
        long DepartureRouteAddressId, long ArrivalRouteAddressId)
    {
        return
            RouteAddressDetails
                .OrderBy(e => e.RouteAddress.Order)
                .SkipWhile(e => e.Id != DepartureRouteAddressId)
                .TakeWhile(e => e.Id != ArrivalRouteAddressId)
                .Aggregate(TimeSpan.Zero,
                    (sum, next) => sum += next.TimeToNextAddress);
    }

    public TimeSpan GetTimeMoving()
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;
        var lastRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).Last().RouteAddressId;

        return GetTimeMoving(firstRouteAddressId, lastRouteAddressId);
    }

    public decimal GetCost(
        long DepartureRouteAddressId, long ArrivalRouteAddressId)
    {
        return
            RouteAddressDetails
                .OrderBy(e => e.RouteAddress.Order)
                .SkipWhile(e => e.Id != DepartureRouteAddressId)
                .TakeWhile(e => e.Id != ArrivalRouteAddressId)
                .Aggregate((decimal)0,
                    (sum, next) => sum += next.CostToNextAddress);
    }

    public decimal GetCost()
    {
        var firstRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).First().RouteAddressId;
        var lastRouteAddressId = RouteAddressDetails
            .OrderBy(e => e.RouteAddress.Order).Last().RouteAddressId;

        return GetCost(firstRouteAddressId, lastRouteAddressId);
    }
}
