using Microsoft.AspNetCore.Mvc;
using Swashbuckle.AspNetCore.Annotations;
using cuqmbr.TravelGuide.Domain.Enums;
using cuqmbr.TravelGuide.Application.TicketGroups;
using cuqmbr.TravelGuide.Application.TicketGroups.Commands.AddTicketGroup;
using cuqmbr.TravelGuide.Application.TicketGroups.ViewModels;
using cuqmbr.TravelGuide.Application.TicketGroups.Models;

namespace cuqmbr.TravelGuide.HttpApi.Controllers;

[Route("ticketGroups")]
public class TicketGroupsController : ControllerBase
{
    [HttpPost]
    [SwaggerOperation("Add a ticketGroup")]
    [SwaggerResponse(
        StatusCodes.Status201Created, "Object successfuly created",
        typeof(TicketGroupDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<ActionResult<TicketGroupDto>> Add(
        [FromBody] AddTicketGroupViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return StatusCode(
            StatusCodes.Status201Created,
            await Mediator.Send(
                new AddTicketGroupCommand()
                {
                    PassangerFirstName = viewModel.PassangerFirstName,
                    PassangerLastName = viewModel.PassangerLastName,
                    PassangerPatronymic = viewModel.PassangerPatronymic,
                    PassangerSex = Sex.FromName(viewModel.PassangerSex),
                    PassangerBirthDate = viewModel.PassangerBirthDate,
                    PurchaseTime = viewModel.PurchaseTime,
                    Status = TicketStatus.FromName(viewModel.Status),
                    Tickets = viewModel.Tickets.Select(e =>
                        new TicketModel()
                        {
                            DepartureRouteAddressGuid = 
                                e.DepartureRouteAddressUuid,
                            ArrivalRouteAddressGuid =
                                e.ArrivalRouteAddressUuid,
                            Order = e.Order,
                            VehicleEnrollmentGuid =
                                e.VehicleEnrollmentUuid,
                        })
                        .ToArray()
                },
                cancellationToken));
    }

    // [HttpGet]
    // [SwaggerOperation("Get a list of all ticketGroups")]
    // [SwaggerResponse(
    //     StatusCodes.Status200OK, "Request successful",
    //     typeof(PaginatedList<TicketGroupDto>))]
    // [SwaggerResponse(
    //     StatusCodes.Status400BadRequest, "Input data validation error",
    //     typeof(HttpValidationProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status403Forbidden,
    //     "Not enough privileges to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status500InternalServerError, "Internal server error",
    //     typeof(ProblemDetails))]
    // public async Task<PaginatedList<TicketGroupDto>> GetPage(
    //     [FromQuery] PageQuery pageQuery, [FromQuery] SearchQuery searchQuery,
    //     [FromQuery] SortQuery sortQuery, 
    //     [FromQuery] GetTicketGroupsPageFilterViewModel filterQuery,
    //     CancellationToken cancellationToken)
    // {
    //     return await Mediator.Send(
    //         new GetTicketGroupsPageQuery()
    //         {
    //             PageNumber = pageQuery.PageNumber,
    //             PageSize = pageQuery.PageSize,
    //             Search = searchQuery.Search,
    //             Sort = sortQuery.Sort,
    //             LongitudeGreaterOrEqualThan = 
    //                 filterQuery.LongitudeGreaterOrEqualThan,
    //             LongitudeLessOrEqualThan = 
    //                 filterQuery.LongitudeLessOrEqualThan,
    //             LatitudeGreaterOrEqualThan = 
    //                 filterQuery.LatitudeGreaterOrEqualThan,
    //             LatitudeLessOrEqualThan = 
    //                 filterQuery.LatitudeLessOrEqualThan,
    //             VehicleType = VehicleType.FromName(filterQuery.VehicleType),
    //             CountryGuid = filterQuery.CountryUuid,
    //             RegionGuid = filterQuery.RegionUuid,
    //             CityGuid = filterQuery.CityUuid
    //         },
    //         cancellationToken);
    // }
    //
    // [HttpGet("{uuid:guid}")]
    // [SwaggerOperation("Get a ticketGroup by uuid")]
    // [SwaggerResponse(
    //     StatusCodes.Status200OK, "Request successful", typeof(TicketGroupDto))]
    // [SwaggerResponse(
    //     StatusCodes.Status400BadRequest, "Input data validation error",
    //     typeof(HttpValidationProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status403Forbidden,
    //     "Not enough privileges to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status404NotFound, "Object not found", typeof(TicketGroupDto))]
    // [SwaggerResponse(
    //     StatusCodes.Status500InternalServerError, "Internal server error",
    //     typeof(ProblemDetails))]
    // public async Task<TicketGroupDto> Get(
    //     [FromRoute] Guid uuid,
    //     CancellationToken cancellationToken)
    // {
    //     return await Mediator.Send(new GetTicketGroupQuery() { Guid = uuid },
    //         cancellationToken);
    // }
    //
    // [HttpPut("{uuid:guid}")]
    // [SwaggerOperation("Update a ticketGroup")]
    // [SwaggerResponse(
    //     StatusCodes.Status200OK, "Request successful", typeof(TicketGroupDto))]
    // [SwaggerResponse(
    //     StatusCodes.Status400BadRequest, "Object already exists",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status400BadRequest, "Input data validation error",
    //     typeof(HttpValidationProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status403Forbidden,
    //     "Not enough privileges to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status404NotFound, "Object not found", typeof(TicketGroupDto))]
    // [SwaggerResponse(
    //     StatusCodes.Status404NotFound, "Parent object not found",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status500InternalServerError, "Internal server error",
    //     typeof(ProblemDetails))]
    // public async Task<TicketGroupDto> Update(
    //     [FromRoute] Guid uuid,
    //     [FromBody] UpdateTicketGroupViewModel viewModel,
    //     CancellationToken cancellationToken)
    // {
    //         return await Mediator.Send(
    //             new UpdateTicketGroupCommand()
    //             {
    //                 Guid = uuid,
    //                 Name = viewModel.Name,
    //                 Longitude = viewModel.Longitude,
    //                 Latitude = viewModel.Latitude,
    //                 VehicleType = VehicleType.FromName(viewModel.VehicleType),
    //                 CityGuid = viewModel.CityUuid
    //             },
    //             cancellationToken);
    // }
    //
    // [HttpDelete("{uuid:guid}")]
    // [SwaggerOperation("Delete a ticketGroup")]
    // [SwaggerResponse(StatusCodes.Status204NoContent, "Request successful")]
    // [SwaggerResponse(
    //     StatusCodes.Status400BadRequest, "Input data validation error",
    //     typeof(HttpValidationProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status403Forbidden,
    //     "Not enough privileges to perform an action",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status404NotFound, "Object not found",
    //     typeof(ProblemDetails))]
    // [SwaggerResponse(
    //     StatusCodes.Status500InternalServerError, "Internal server error",
    //     typeof(ProblemDetails))]
    // public async Task<IActionResult> Delete(
    //     [FromRoute] Guid uuid,
    //     CancellationToken cancellationToken)
    // {
    //     await Mediator.Send(
    //         new DeleteTicketGroupCommand() { Guid = uuid },
    //         cancellationToken);
    //     return StatusCode(StatusCodes.Status204NoContent);
    // }
}
