using MediatR;
using cuqmbr.TravelGuide.Application.TicketGroups
    .Commands.RemoveOldReservedTicketGroups;

namespace cuqmbr.TravelGuide.HttpApi.HostedServices;

public class ReservedTicketRemoverHostedService : BackgroundService
{
    private Timer _timer = null;

    public ReservedTicketRemoverHostedService(
        IServiceProvider serviceProvider)
    {
        ServiceProvider = serviceProvider;
    }

    public IServiceProvider ServiceProvider;

    protected override async Task ExecuteAsync(CancellationToken cancellationToken)
    {
        _timer = new Timer(async (state) =>
            {
                using var scope = ServiceProvider.CreateScope();
                var _mediator = scope.ServiceProvider
                    .GetRequiredService<IMediator>();

                await _mediator.Send(
                    new RemoveOldReservedTicketGroupsCommand()
                    {
                        ReservedFor = TimeSpan.FromMinutes(10)
                    }, cancellationToken);
            },
            null, TimeSpan.Zero, TimeSpan.FromMinutes(1));
    }

    public override async Task StopAsync(CancellationToken cancellationToken)
    {
        _timer?.Dispose();
        await base.StopAsync(cancellationToken);
    }
}
