using System.IdentityModel.Tokens.Jwt;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;

namespace cuqmbr.TravelGuide.HttpApi.Services;

public sealed class AspNetSessionUserService : SessionUserService
{
    private readonly HttpContext _httpContext;

    public AspNetSessionUserService(IHttpContextAccessor httpContextAccessor)
    {
        _httpContext = httpContextAccessor.HttpContext;
    }

    public Guid? Guid
    {
        get
        {
            var claimValue = _httpContext.User.Claims
                .FirstOrDefault(c => c.Properties
                    .Any(p => p.Value == JwtRegisteredClaimNames.Sub))
                ?.Value;

            var parsed = System.Guid.TryParse(claimValue, out var guid);

            return parsed ? guid : null;
        }
    }

    public string? Username => _httpContext.User.Claims
        .FirstOrDefault(c => c.Properties
            .Any(p => p.Value == JwtRegisteredClaimNames.Nickname))
        ?.Value;

    public string? Email => _httpContext.User.Claims
        .FirstOrDefault(c => c.Properties
            .Any(p => p.Value == JwtRegisteredClaimNames.Email))
        ?.Value;

    public ICollection<IdentityRole> Roles => _httpContext.User.Claims
        .Where(c => c.Properties
            .Any(p => p.Value == "roles"))
        .Select(c => IdentityRole.FromName(c.Value))
        .ToArray() ?? default!;

    public string? AccessToken =>
        _httpContext.Request.Cookies["accessToken"] ??
        _httpContext.Request.Headers["Authorization"]
            .ToString()?.Replace("Bearer ", "");

    public string? RefreshToken =>
        _httpContext.Request.Cookies["refreshToken"];
}
