using Microsoft.OpenApi.Models;
using Swashbuckle.AspNetCore.SwaggerGen;

namespace cuqmbr.TravelGuide.HttpApi.Swashbuckle.OperationFilters;

public class AcceptCurrencyHeaderOperationFilter : IOperationFilter
{
    public void Apply(OpenApiOperation operation, OperationFilterContext context)
    {
        // TODO: Remove security requirements
        operation.Security ??= new List<OpenApiSecurityRequirement>();

        var acceptCurrency = new OpenApiSecurityScheme
        {
            Reference = new OpenApiReference
            {
                Type = ReferenceType.SecurityScheme,
                Id = "Accept-Currency"
            }
        };

        operation.Security.Add(new OpenApiSecurityRequirement
        {
            [acceptCurrency] = new List<string>()
        });

        if (operation.Parameters == null)
            operation.Parameters = new List<OpenApiParameter>();

        operation.Parameters.Add(new OpenApiParameter
        {
            Name = "Accept-Currency",
            Description = "ISO-4217 Currency Code.",
            In = ParameterLocation.Header,
            Schema = new OpenApiSchema { Type = "String" },
            Required = false
        });
    }
}
