using System.Dynamic;
using System.Security.Cryptography;
using System.Text;
using cuqmbr.TravelGuide.Domain.Enums;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace cuqmbr.TravelGuide.Infrastructure.Services;

public sealed class LiqPayPaymentService :
    cuqmbr.TravelGuide.Application.Common.Services.LiqPayPaymentService
{
    private readonly LiqPayConfigurationOptions _configuration;
    private readonly string _callbackAddressBase;
    private readonly string _resultAddressBase;

    private readonly IHttpClientFactory _httpClientFactory;

    public LiqPayPaymentService(
        IOptions<ConfigurationOptions> configurationOptions,
        IHttpClientFactory httpClientFactory)
    {
        _configuration = configurationOptions.Value.PaymentProcessing.LiqPay;
        _callbackAddressBase =
            configurationOptions.Value.PaymentProcessing.CallbackAddressBase;
        _resultAddressBase =
            configurationOptions.Value.PaymentProcessing.ResultAddressBase;
    }

    public string GetPaymentLink(
        decimal amount, Currency currency,
        string orderId, TimeSpan validity, string description,
        string resultPath, string callbackPath)
    {
        dynamic request = new ExpandoObject();

        request.version = 3;
        request.public_key = _configuration.PublicKey;
        request.action = "pay";
        request.amount = amount;
        request.currency = currency.Name.ToUpper();
        request.description = description;
        request.order_id = orderId;
        request.expired_date = DateTimeOffset.UtcNow.Add(validity)
            .ToString("yyyy-MM-dd HH:mm:ss");
        request.result_url = $"{_resultAddressBase}{resultPath}";
        request.server_url = $"{_callbackAddressBase}{callbackPath}";

        var requestJsonString = (string)JsonConvert.SerializeObject(request);


        var requestJsonStringBytes = Encoding.UTF8.GetBytes(requestJsonString);

        var data = Convert.ToBase64String(requestJsonStringBytes);

        var signature = Convert.ToBase64String(SHA1.HashData(
            Encoding.UTF8.GetBytes(
                _configuration.PrivateKey +
                data +
                _configuration.PrivateKey)));


        return
            "https://www.liqpay.ua/api/3/checkout" +
            $"?data={data}&signature={signature}";
    }

    public bool IsValidSignature(string postData, string postSignature)
    {
        var signature = Convert.ToBase64String(SHA1.HashData(
            Encoding.UTF8.GetBytes(
                _configuration.PrivateKey +
                postData +
                _configuration.PrivateKey)));

        return postSignature.Equals(signature);
    }
}
