using System.Security.Cryptography;
using cuqmbr.TravelGuide.Application.Common.Services;

namespace cuqmbr.TravelGuide.Infrastructure.Services;

public sealed class Pbkdf2PasswordHasherService : PasswordHasherService
{
    private const int IterationCount = 210_000;

    private readonly HashAlgorithmName Hash = HashAlgorithmName.SHA3_512;
    private const byte HashLengthBytes = 64;
    
    public Task<byte[]> HashAsync(byte[] password, byte[] salt,
        CancellationToken cancellationToken)
    {
        return Task.FromResult(
            Rfc2898DeriveBytes.Pbkdf2(
                password, salt, IterationCount,
                Hash, HashLengthBytes));
    }

    public Task<bool> IsValidHashAsync(byte[] hash, byte[] password,
        byte[] salt, CancellationToken cancellationToken)
    {
        var computedHash = Rfc2898DeriveBytes.Pbkdf2(
                password, salt, IterationCount,
                Hash, HashLengthBytes);

        return Task.FromResult(
            Enumerable.SequenceEqual(computedHash, hash));
    }
}
