using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Persistence.Repositories;
using cuqmbr.TravelGuide.Persistence.InMemory.Repositories;

namespace cuqmbr.TravelGuide.Persistence.InMemory;

public sealed class InMemoryUnitOfWork : UnitOfWork
{
    private readonly InMemoryDbContext _dbContext;

    public InMemoryUnitOfWork(
        InMemoryDbContext dbContext)
    {
        _dbContext = dbContext;

        CountryRepository = new InMemoryCountryRepository(_dbContext);
        RegionRepository = new InMemoryRegionRepository(_dbContext);
        CityRepository = new InMemoryCityRepository(_dbContext);
        AddressRepository = new InMemoryAddressRepository(_dbContext);
        RouteRepository = new InMemoryRouteRepository(_dbContext);
        VehicleRepository = new InMemoryVehicleRepository(_dbContext);
        BusRepository = new InMemoryBusRepository(_dbContext);
        AircraftRepository = new InMemoryAircraftRepository(_dbContext);
        TrainRepository = new InMemoryTrainRepository(_dbContext);
        VehicleEnrollmentRepository =
            new InMemoryVehicleEnrollmentRepository(_dbContext);
        RouteAddressRepository =
            new InMemoryRouteAddressRepository(_dbContext);
        CompanyRepository = new InMemoryCompanyRepository(_dbContext);
        EmployeeRepository = new InMemoryEmployeeRepository(_dbContext);
        TicketGroupRepository = new InMemoryTicketGroupRepository(_dbContext);
        TicketRepository = new InMemoryTicketRepository(_dbContext);
        RouteAddressDetailRepository =
            new InMemoryRouteAddressDetailRepository(_dbContext);
        VehicleEnrollmentEmployeeRepository =
            new InMemoryVehicleEnrollmentEmployeeRepository(_dbContext);

        AccountRepository = new InMemoryAccountRepository(_dbContext);
        RoleRepository = new InMemoryRoleRepository(_dbContext);
        AccountRoleRepository = new InMemoryAccountRoleRepository(_dbContext);
        RefreshTokenRepository = new InMemoryRefreshTokenRepository(_dbContext);
    }

    public CountryRepository CountryRepository { get; init; }

    public RegionRepository RegionRepository { get; init; }

    public CityRepository CityRepository { get; init; }

    public AddressRepository AddressRepository { get; init; }

    public RouteRepository RouteRepository { get; init; }

    public VehicleRepository VehicleRepository { get; init; }

    public BusRepository BusRepository { get; init; }

    public AircraftRepository AircraftRepository { get; init; }

    public TrainRepository TrainRepository { get; init; }

    public VehicleEnrollmentRepository VehicleEnrollmentRepository { get; init; }

    public RouteAddressRepository RouteAddressRepository { get; init; }

    public CompanyRepository CompanyRepository { get; init; }

    public EmployeeRepository EmployeeRepository { get; init; }

    public TicketGroupRepository TicketGroupRepository { get; init; }

    public TicketRepository TicketRepository { get; init; }

    public RouteAddressDetailRepository RouteAddressDetailRepository { get; init; }

    public VehicleEnrollmentEmployeeRepository VehicleEnrollmentEmployeeRepository { get; init; }


    public AccountRepository AccountRepository { get; init; }

    public RoleRepository RoleRepository { get; init; }

    public AccountRoleRepository AccountRoleRepository { get; init; }

    public RefreshTokenRepository RefreshTokenRepository { get; init; }

    public int Save()
    {
        return _dbContext.SaveChanges();
    }

    public async Task<int> SaveAsync(CancellationToken cancellationToken)
    {
        return await _dbContext.SaveChangesAsync(cancellationToken);
    }

    public void Dispose()
    {
        Dispose(disposing: true);
        GC.SuppressFinalize(this);
    }

    public void Dispose(bool disposing)
    {
        if (disposing)
        {
            _dbContext.Dispose();
        }
    }
}
