using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class AccountConfiguration : BaseConfiguration<Account>
{
    public override void Configure(EntityTypeBuilder<Account> builder)
    {
        builder
            .ToTable("accounts");

        base.Configure(builder);


        builder
            .Property(a => a.Username)
            .HasColumnName("username")
            .HasColumnType("varchar(32)")
            .IsRequired(true);

        builder
            .Property(a => a.Email)
            .HasColumnName("email")
            .HasColumnType("varchar(256)")
            .IsRequired(true);

        // Base64 encoded PBKDF2 SHA512.
        // Base64 represents 3 input bytes 4 output bytes, it adds padding
        // if the input bytes count isn't divisible by 3.
        // 512 / 8 / 3 = 21.(3) => 22 input characters (with padding) =>
        // 88 output characters.
        builder
            .Property(a => a.PasswordHash)
            .HasColumnName("password_hash")
            .HasColumnType("varchar(88)")
            .IsRequired(true);

        // Base64 encoded 128 bits 
        // 128 / 8 / 3 = 5.(3) => 6 in chars \w padding => 24 out chars
        builder
            .Property(a => a.PasswordSalt)
            .HasColumnName("password_salt")
            .HasColumnType("varchar(24)")
            .IsRequired(true);
    }
}
