using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class VehicleEnrollmentEmployeeConfiguration :
    BaseConfiguration<VehicleEnrollmentEmployee>
{
    public override void Configure(
        EntityTypeBuilder<VehicleEnrollmentEmployee> builder)
    {
        builder
            .ToTable("vehicle_enrollment_employees");

        base.Configure(builder);


        builder
            .Property(vee => vee.EmployeeId)
            .HasColumnName("employee_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(vee => vee.Employee)
            .WithMany(e => e.VehicleEnrollmentEmployees)
            .HasForeignKey(vee => vee.EmployeeId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(vee => vee.EmployeeId)
                        .Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(vee => vee.EmployeeId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(vee => vee.EmployeeId)
                        .Metadata.GetColumnName()}");


        builder
            .Property(vee => vee.VehicleEnrollmentId)
            .HasColumnName("vehicle_enrollment_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(vee => vee.VehicleEnrollment)
            .WithMany(ve => ve.VehicleEnrollmentEmployees)
            .HasForeignKey(vee => vee.VehicleEnrollmentId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(vee => vee.VehicleEnrollmentId)
                        .Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(vee => vee.VehicleEnrollmentId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(vee => vee.VehicleEnrollmentId)
                        .Metadata.GetColumnName()}");
    }
}
