﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Add_Employee_and_EmployeeDocument : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateSequence(
                name: "employee_documents_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "employees_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "employees",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.employees_id_sequence')"),
                    first_name = table.Column<string>(type: "varchar(32)", nullable: false),
                    last_name = table.Column<string>(type: "varchar(32)", nullable: false),
                    patronymic = table.Column<string>(type: "varchar(32)", nullable: false),
                    sex = table.Column<string>(type: "varchar(32)", nullable: false),
                    birth_date = table.Column<DateOnly>(type: "date", nullable: false),
                    company_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_employees", x => x.id);
                    table.UniqueConstraint("altk_employees_uuid", x => x.uuid);
                    table.CheckConstraint("ck_employees_sex", "sex IN ('male', 'female')");
                    table.ForeignKey(
                        name: "fk_employees_company_id",
                        column: x => x.company_id,
                        principalSchema: "application",
                        principalTable: "companies",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "employee_documents",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.employee_documents_id_sequence')"),
                    document_type = table.Column<string>(type: "varchar(64)", nullable: false),
                    information = table.Column<string>(type: "varchar(256)", nullable: false),
                    employee_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_employee_documents", x => x.id);
                    table.UniqueConstraint("altk_employee_documents_uuid", x => x.uuid);
                    table.CheckConstraint("ck_employee_documents_document_type", "document_type IN ('passport')");
                    table.ForeignKey(
                        name: "fk_employee_documents_employee_id",
                        column: x => x.employee_id,
                        principalSchema: "application",
                        principalTable: "employees",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "ix_employee_documents_employee_id",
                schema: "application",
                table: "employee_documents",
                column: "employee_id");

            migrationBuilder.CreateIndex(
                name: "ix_employees_company_id",
                schema: "application",
                table: "employees",
                column: "company_id");
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "employee_documents",
                schema: "application");

            migrationBuilder.DropTable(
                name: "employees",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "employee_documents_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "employees_id_sequence",
                schema: "application");
        }
    }
}
