﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Add_Ticket_and_TicketGroup : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateSequence(
                name: "ticket_groups_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "tickets_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "ticket_groups",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.ticket_groups_id_sequence')"),
                    passanger_first_name = table.Column<string>(type: "varchar(32)", nullable: false),
                    passanger_last_name = table.Column<string>(type: "varchar(32)", nullable: false),
                    passanger_patronymic = table.Column<string>(type: "varchar(32)", nullable: false),
                    passanger_sex = table.Column<string>(type: "varchar(32)", nullable: false),
                    passanger_birth_date = table.Column<DateOnly>(type: "date", nullable: false),
                    purchase_time = table.Column<DateTimeOffset>(type: "timestamptz", nullable: false),
                    returned = table.Column<bool>(type: "boolean", nullable: false),
                    travel_time = table.Column<TimeSpan>(type: "interval", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_ticket_groups", x => x.id);
                    table.UniqueConstraint("altk_ticket_groups_uuid", x => x.uuid);
                    table.CheckConstraint("ck_ticket_groups_passanger_sex", "passanger_sex IN ('male', 'female')");
                });

            migrationBuilder.CreateTable(
                name: "tickets",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.tickets_id_sequence')"),
                    departure_route_address_id = table.Column<long>(type: "bigint", nullable: false),
                    arrival_route_address_id = table.Column<long>(type: "bigint", nullable: false),
                    order = table.Column<short>(type: "smallint", nullable: false),
                    currency = table.Column<string>(type: "varchar(8)", nullable: false),
                    cost = table.Column<decimal>(type: "numeric(24,12)", nullable: false),
                    ticket_group_id = table.Column<long>(type: "bigint", nullable: false),
                    vehicle_enrollment_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_tickets", x => x.id);
                    table.UniqueConstraint("altk_tickets_uuid", x => x.uuid);
                    table.CheckConstraint("ck_tickets_currency", "currency IN ('DEFAULT', 'USD', 'EUR', 'UAH')");
                    table.ForeignKey(
                        name: "FK_tickets_route_addresses_arrival_route_address_id",
                        column: x => x.arrival_route_address_id,
                        principalSchema: "application",
                        principalTable: "route_addresses",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_tickets_route_addresses_departure_route_address_id",
                        column: x => x.departure_route_address_id,
                        principalSchema: "application",
                        principalTable: "route_addresses",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_tickets_ticket_group_id",
                        column: x => x.ticket_group_id,
                        principalSchema: "application",
                        principalTable: "ticket_groups",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_tickets_vehicle_enrollment_id",
                        column: x => x.vehicle_enrollment_id,
                        principalSchema: "application",
                        principalTable: "vehicle_enrollments",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "IX_tickets_arrival_route_address_id",
                schema: "application",
                table: "tickets",
                column: "arrival_route_address_id");

            migrationBuilder.CreateIndex(
                name: "IX_tickets_departure_route_address_id",
                schema: "application",
                table: "tickets",
                column: "departure_route_address_id");

            migrationBuilder.CreateIndex(
                name: "ix_tickets_ticket_group_id",
                schema: "application",
                table: "tickets",
                column: "ticket_group_id");

            migrationBuilder.CreateIndex(
                name: "ix_tickets_vehicle_enrollment_id",
                schema: "application",
                table: "tickets",
                column: "vehicle_enrollment_id");
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "tickets",
                schema: "application");

            migrationBuilder.DropTable(
                name: "ticket_groups",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "ticket_groups_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "tickets_id_sequence",
                schema: "application");
        }
    }
}
