﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Add_Account_Role_and_AccountRole : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateSequence(
                name: "account_roles_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "accounts_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "refresh_tokens_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "roles_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "accounts",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.accounts_id_sequence')"),
                    username = table.Column<string>(type: "varchar(32)", nullable: false),
                    email = table.Column<string>(type: "varchar(256)", nullable: false),
                    password_hash = table.Column<string>(type: "varchar(88)", nullable: false),
                    password_salt = table.Column<string>(type: "varchar(24)", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_accounts", x => x.id);
                    table.UniqueConstraint("altk_accounts_uuid", x => x.uuid);
                });

            migrationBuilder.CreateTable(
                name: "roles",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.roles_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_roles", x => x.id);
                    table.UniqueConstraint("altk_roles_uuid", x => x.uuid);
                    table.CheckConstraint("ck_roles_name", "name IN ('administrator', 'user', 'company_owner', 'company_employee')");
                });

            migrationBuilder.CreateTable(
                name: "refresh_tokens",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.refresh_tokens_id_sequence')"),
                    value = table.Column<string>(type: "varchar(24)", nullable: false),
                    creation_time = table.Column<DateTimeOffset>(type: "timestamptz", nullable: false),
                    expiration_time = table.Column<DateTimeOffset>(type: "timestamptz", nullable: false),
                    revocation_time = table.Column<DateTimeOffset>(type: "timestamptz", nullable: true),
                    account_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_refresh_tokens", x => x.id);
                    table.UniqueConstraint("altk_refresh_tokens_uuid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_refresh_tokens_account_id",
                        column: x => x.account_id,
                        principalSchema: "application",
                        principalTable: "accounts",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "account_roles",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.account_roles_id_sequence')"),
                    account_id = table.Column<long>(type: "bigint", nullable: false),
                    role_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_account_roles", x => x.id);
                    table.UniqueConstraint("altk_account_roles_uuid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_account_roles_account_id",
                        column: x => x.account_id,
                        principalSchema: "application",
                        principalTable: "accounts",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_account_roles_role_id",
                        column: x => x.role_id,
                        principalSchema: "application",
                        principalTable: "roles",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "ix_account_roles_account_id",
                schema: "application",
                table: "account_roles",
                column: "account_id");

            migrationBuilder.CreateIndex(
                name: "ix_account_roles_role_id",
                schema: "application",
                table: "account_roles",
                column: "role_id");

            migrationBuilder.CreateIndex(
                name: "ix_refresh_tokens_account_id",
                schema: "application",
                table: "refresh_tokens",
                column: "account_id");
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "account_roles",
                schema: "application");

            migrationBuilder.DropTable(
                name: "refresh_tokens",
                schema: "application");

            migrationBuilder.DropTable(
                name: "roles",
                schema: "application");

            migrationBuilder.DropTable(
                name: "accounts",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "account_roles_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "accounts_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "refresh_tokens_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "roles_id_sequence",
                schema: "application");
        }
    }
}
