using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Persistence.Repositories;
using cuqmbr.TravelGuide.Persistence.PostgreSql.Repositories;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql;

public sealed class PostgreSqlUnitOfWork : UnitOfWork
{
    private readonly PostgreSqlDbContext _dbContext;

    public PostgreSqlUnitOfWork(
        PostgreSqlDbContext dbContext)
    {
        _dbContext = dbContext;

        CountryRepository = new PostgreSqlCountryRepository(_dbContext);
        RegionRepository = new PostgreSqlRegionRepository(_dbContext);
        CityRepository = new PostgreSqlCityRepository(_dbContext);
        AddressRepository = new PostgreSqlAddressRepository(_dbContext);
        RouteRepository = new PostgreSqlRouteRepository(_dbContext);
        VehicleRepository = new PostgreSqlVehicleRepository(_dbContext);
        BusRepository = new PostgreSqlBusRepository(_dbContext);
        AircraftRepository = new PostgreSqlAircraftRepository(_dbContext);
        TrainRepository = new PostgreSqlTrainRepository(_dbContext);
        VehicleEnrollmentRepository =
            new PostgreSqlVehicleEnrollmentRepository(_dbContext);
        RouteAddressRepository =
            new PostgreSqlRouteAddressRepository(_dbContext);
        CompanyRepository = new PostgreSqlCompanyRepository(_dbContext);
        EmployeeRepository = new PostgreSqlEmployeeRepository(_dbContext);
        TicketGroupRepository = new PostgreSqlTicketGroupRepository(_dbContext);
        TicketRepository = new PostgreSqlTicketRepository(_dbContext);
        RouteAddressDetailRepository =
            new PostgreSqlRouteAddressDetailRepository(_dbContext);
        VehicleEnrollmentEmployeeRepository =
            new PostgreSqlVehicleEnrollmentEmployeeRepository(_dbContext);

        AccountRepository = new PostgreSqlAccountRepository(_dbContext);
        RoleRepository = new PostgreSqlRoleRepository(_dbContext);
        AccountRoleRepository = new PostgreSqlAccountRoleRepository(_dbContext);
        RefreshTokenRepository = new PostgreSqlRefreshTokenRepository(_dbContext);
    }

    public CountryRepository CountryRepository { get; init; }

    public RegionRepository RegionRepository { get; init; }

    public CityRepository CityRepository { get; init; }

    public AddressRepository AddressRepository { get; init; }

    public RouteRepository RouteRepository { get; init; }

    public VehicleRepository VehicleRepository { get; init; }

    public BusRepository BusRepository { get; init; }

    public AircraftRepository AircraftRepository { get; init; }

    public TrainRepository TrainRepository { get; init; }

    public VehicleEnrollmentRepository VehicleEnrollmentRepository { get; init; }

    public RouteAddressRepository RouteAddressRepository { get; init; }

    public CompanyRepository CompanyRepository { get; init; }

    public EmployeeRepository EmployeeRepository { get; init; }

    public TicketGroupRepository TicketGroupRepository { get; init; }

    public TicketRepository TicketRepository { get; init; }

    public RouteAddressDetailRepository RouteAddressDetailRepository { get; init; }

    public VehicleEnrollmentEmployeeRepository VehicleEnrollmentEmployeeRepository { get; init; }


    public AccountRepository AccountRepository { get; init; }

    public RoleRepository RoleRepository { get; init; }

    public AccountRoleRepository AccountRoleRepository { get; init; }

    public RefreshTokenRepository RefreshTokenRepository { get; init; }

    public int Save()
    {
        return _dbContext.SaveChanges();
    }

    public async Task<int> SaveAsync(CancellationToken cancellationToken)
    {
        return await _dbContext.SaveChangesAsync(cancellationToken);
    }

    public void Dispose()
    {
        Dispose(disposing: true);
        GC.SuppressFinalize(this);
    }

    public void Dispose(bool disposing)
    {
        if (disposing)
        {
            _dbContext.Dispose();
        }
    }
}
